<?php

class Optimal_Shipping_Model_Carrier
    extends Mage_Shipping_Model_Carrier_Abstract
    implements Mage_Shipping_Model_Carrier_Interface
{
    /**
     * Carrier's code, as defined in parent class
     *
     * @var string
     */
    protected $_code = 'optimal_shipping';

    /**
     * Returns available shipping rates for Optimal Shipping carrier
     *
     * @param Mage_Shipping_Model_Rate_Request $request
     * @return Mage_Shipping_Model_Rate_Result
     */
    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        /** @var Mage_Shipping_Model_Rate_Result $result */
        $result = Mage::getModel('shipping/rate_result');

        /** @var Optimal_Shipping_Helper_Data $expressMaxProducts */
        $expressMaxWeight = Mage::helper('optimal_shipping')->getExpressMaxWeight();

        $expressAvailable = true;
        foreach ($request->getAllItems() as $item) {
            if ($item->getWeight() > $expressMaxWeight) {
                $expressAvailable = false;
            }
        }

        if ($expressAvailable) {
            $result->append($this->_getExpressRate($request));
        }

        return $result;
    }

    /**
     * Returns Allowed shipping methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        return array(
            'express'     =>  'Express delivery',
        );
    }

    /**
     * Get Express rate object
     *
     * @return Mage_Shipping_Model_Rate_Result_Method
     */
    protected function _getExpressRate($request)
    {

        $dest_country = $request->getDestCountryId(); 
        $dest_post = $request->getDestPostcode(); 
        $dest_city = $request->getDestCity();        
        $weight = 0;

        foreach($request->getAllItems() as $item){
            $weight += $item->getWeight()*$item->getQty();
        }

        $account = ($this->getConfigData('account'));
        $origin = $this->getShippingOrigin();
        $increase = $this->getConfigData('surcharge');
        $url = "https://www.optimalship.com/rates/rates.php?acc=" . $account . 
        "&gtw=" . $origin['country_id'] .
        "&dst=" . $dest_country .
        "&wgt=" . $weight .
        "&prd=P&cty=" . $dest_city .
        "&zip=" . $origin['postcode'] .
        "&pst=" . $dest_post .
        "&schgtype=PCT&schgbort=B&schgamnt=" . $increase .
        "&json=Y";
        $url = str_replace(' ','%20',$url);
        $json = file_get_contents($url);
        $obj = json_decode($json,true);
        $price = $obj["ratingResponse"]["totalCharge"];

        /** @var Mage_Shipping_Model_Rate_Result_Method $rate */
        $rate = Mage::getModel('shipping/rate_result_method');
        $rate->setCarrier($this->_code);
        $rate->setCarrierTitle($this->getConfigData('title'));
        $rate->setMethod('express');
        $rate->setMethodTitle('Express delivery');
        $rate->setPrice($price);
        $rate->setCost(0);

        return $rate;
    }

    protected function getShippingOrigin()
    {
        /**
         * Get Shipping origin data from store scope config
         */ 

        return [
            'country_id' => Mage::getStoreConfig('shipping/origin/country_id'),
            'region_id' => Mage::getStoreConfig('shipping/origin/region_id'),
            'postcode' => Mage::getStoreConfig('shipping/origin/postcode'),
            'city' => Mage::getStoreConfig('shipping/origin/city')
        ];
    }
}